#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#
import os
import sys
import socket

import Usb
import asset

# -----------------------------------------------------------------------------------
# --- Global options ...
# -----------------------------------------------------------------------------------
synoConfig  = '/etc/synoinfo.conf'
upsConfig   = '/usr/syno/etc/ups/synoups.conf'

HostIP      = '127.0.0.1'
HostPort    = 3493
SockTimeOut = 1.0

iStatus     = {   'OL'         : 'Online',                     # 'green'
                  'OB DISCHRG' : 'Batterie',                   # 'red'
                  'OL CHRG'    : 'Laden',                      # 'orange'
            #     'OL CHRG'    : 'Laden bald abgeschlossen',   # 'yellowgreen'
              }

iUsbKey     = [   'supportups',
                  'ups_enabled',
                  'ups_mode',
                  'upsslave_server',
                  'upsslave_enabled',
                  'ups_acl',
              ]

# -----------------------------------------------------------------------
class BaseException( Exception ) :
   """Base exception for this Module"""
   # --- "Exception" is a Built-in Exceptions

# -----------------------------------------------------------------------
class nutException( BaseException ) :
   """Exception handler in Module: NUT"""

# -----------------------------------------------------------------------------------
# --- Helper function for getting the current exception in a try/except block
# -----------------------------------------------------------------------------------
def getException() :

   return sys.exc_info()[1]

# --------------------------------------------------------------------------
# --- Get Syno UPS-Settings ...
# --------------------------------------------------------------------------
def synoUsvNice() :

   sRet   = dict()
   iLines = []

   # -----------------------------------------------------------------------
   # --- Get '/etc/synoinfo.conf' ...
   with open( synoConfig, 'r', encoding='utf-8' ) as f:
      iRAW = f.read().strip().split( '\n' )

   for sLine in iRAW :
      for sID in iUsbKey :
         if not sID in sLine : continue

         sName, sValue = sLine.strip().split( '=', 1 )

         # --- Cosmetics only ...
         sName = sName.replace( 'ups', ' UPS ' )
         sName = sName.replace( '_', ' ' )
         sName = sName[:1].upper() + sName[1:]
         sName = ' '.join( sName.split() )

         # --- What we want ...
         sRet[ sName ] = sValue.strip( '"' )

   # -----------------------------------------------------------------------
   # --- Get '/usr/syno/etc/synoups.conf' ...
   if asset.f_Dsm7 :
      with open( upsConfig, 'r', encoding='utf-8' ) as f:
         iRAW = f.read().strip().split( '\n' )

      for sLine in iRAW :
         for sID in iUsbKey :
            if not sID in sLine : continue

            sName, sValue = sLine.strip().split( '=', 1 )

            # --- Cosmetics only ...
            sName = sName.replace( 'ups', ' UPS ' )
            sName = sName.replace( '_', ' ' )
            sName = sName[:1].upper() + sName[1:]
            sName = ' '.join( sName.split() )

            # --- What we want ...
            sRet[ sName ] = sValue.strip( '"' )

   # -----------------------------------------------------------------------
   if 'UPS acl' in sRet : sRet['UPS acl'] = ' '.join( sRet['UPS acl'].replace( '|', ' ').split() )

   # -----------------------------------------------------------------------
   return sRet

# --------------------------------------------------------------------------
# --- Query "UPS Status" informations via socket "127.0.0.1:3493"
# --- Same as Bash : upsc ups
# --------------------------------------------------------------------------
def getUsvNice( opt=False ) :

   global HostIP
   sock  = False
   sSyno = synoUsvNice()

   try :
      # -----------------------------------------------------------------------
      if sSyno['Support UPS'] != 'yes' :
         raise nutException( 'No UPS support on this device.' )

      # -----------------------------------------------------------------------
      if sSyno['UPS mode'] != 'usb' and sSyno['UPS mode'] != 'slave' :
         raise nutException( 'Unsuported UPS-Mode :', sSyno['UPS mode'] )

      # -----------------------------------------------------------------------
      if sSyno['UPS mode'] == 'slave' :
         if sSyno['UPS slave enabled'] != 'yes' :
            raise nutException( 'UPS support is disabled.' )
         else :
            HostIP = sSyno['UPS slave server']
      else :
         if sSyno['UPS enabled'] != 'yes' :
            raise nutException( 'UPS support is disabled.' )

   except :
      # -----------------------------------------------------------------------
      raise nutException( 'Error occurred during UPS support detection :\n{0}'.format( getException() ) )

   # -----------------------------------------------------------------------
   # --- Create a socket object ...
   try :
      sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
   except OSError :
      raise nutException( 'Socket Error :\n{0}'.format( getException() ) )

   if not sock :
      raise nutException( 'Socket creation failed ...' )

   # -----------------------------------------------------------------------
   # --- Connection to Host-IP on the port with TimeOut ...
   try :
      sock.settimeout( SockTimeOut )
      sock.connect( ( HostIP, HostPort ) )
   except OSError :
      raise nutException( 'Socket Error : {0} / {1}:{2}'.format( getException(), HostIP, HostPort ) )

   # -----------------------------------------------------------------------
   # --- Initiate query of the UPS parameter / information
   sock.sendall( b'LIST VAR ups\nLOGOUT\n' )

   # -----------------------------------------------------------------------
   # --- Start reading UPS-Informations ...
   bRet = bytearray()

   try :
      while True:
         bPack = sock.recv( 128 )
         if not bPack : break

         bRet.extend( bPack )
   except OSError :
      sock.close()
      raise nutException( 'Socket read Error : {0}'.format( getException() ) )

   # -----------------------------------------------------------------------
   # --- Close socket object ...
   sock.close()

   # -----------------------------------------------------------------------
   # --- Decode "Byte"-Responce to 'utf-8' ...
   bRet = bRet.decode( 'utf-8' ).strip().split( '\n' )

   # -----------------------------------------------------------------------
   # --- Check that the responce was complete and correct ...
   if bRet :
      if bRet[0].startswith( 'ERR' ) :
         raise nutException( 'Query faild : "%s"' % bRet[0] )

      if not bRet[0].startswith( 'BEGIN LIST VAR' ) :
         raise nutException( 'Query faild : Wrong response header "%s"' % bRet[0] )

      if not bRet[-1].startswith( 'OK Goodbye' ) :
         raise nutException( 'Query faild : Wrong "LOGOUT" response "%s"' % bRet[-1] )

      if not bRet[-2].startswith( 'END LIST VAR' ) :
         msg =( 'Query faild : Incomplete response received :\n' )
         for sLine in bRet :
            msg += sLine + '\n'
         raise nutException( msg )
   else :
      raise nutException( 'Query faild : Empty response received' )

   # -----------------------------------------------------------------------
   # --- Remove Headline / Footer / Headers ('VAR ups') ...
   iRet = [ x.split( 'VAR ups ', 1 )[-1] for x in bRet[1:-2] ]

   # -----------------------------------------------------------------------
   # --- Separate Responce into Dict ...
   sRet = dict()

   for sLine in iRet :
      try :
         sKey, sValue = sLine.split( ' ', 1 )

         # --- Reformat / clean Value & Key (Name) ...
         sValue = ' '.join( sValue.strip( '"' ).split() )
         sKey   = ' '.join( sKey.replace( '.', ' ' ).split() )

         sPart, sKey = sKey.split( ' ', 1 )

         # --- Cosmetics only ...
         sPart = sPart.capitalize()
         sKey  = sKey.capitalize()

         try    : sRet[ sPart ].update( { sKey : sValue } )
         except : sRet[ sPart ] = { sKey : sValue }
      except :
         # --- On Error skip line (continue) ...
         pass

   # -----------------------------------------------------------------------
   # --- Get Vendor- / Product-ID ...
   f = Usb.ParseUsbIDs()
   sVendor  = ''
   sProduct = ''

   if 'Ups' in sRet and 'Vendorid' in sRet['Ups'] :
      sVendor = f.FindUsbVendor( sRet['Ups']['Vendorid'] )

      if 'Productid' in sRet['Ups'] :
         sProduct = f.FindUsbProduct( sRet['Ups']['Vendorid'], sRet['Ups']['Productid'] )

      if sVendor  : sRet['Ups']['Vendorid' ] = '{0} [{1}]'.format( sRet['Ups']['Vendorid' ].upper(), sVendor  )
      if sProduct : sRet['Ups']['Productid'] = '{0} [{1}]'.format( sRet['Ups']['Productid'].upper(), sProduct )

   # -----------------------------------------------------------------------
   # --- Add syno-UPS settings ...
   # -----------------------------------------------------------------------
   sRet['syno'] = sSyno 

   # -----------------------------------------------------------------------
   return sRet

# --------------------------------------- END ---------------------------------------
# -----------------------------------------------------------------------------------
#  sys.exit()
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
