# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#

import os
import sys
import shutil
import glob
# import time
import contextlib
from   string    import Formatter
from   html      import escape
from   functools import wraps
# import errno

import asset
import lib

# -----------------------------------------------------------------------------------
# --- Wrapper to prevent : [Errno 32] Broken pipe
# --> https://stackoverflow.com/questions/14207708/ioerror-errno-32-broken-pipe-python
# -----------------------------------------------------------------------------------
def BrokenPipe( f ) :
   @wraps( f )
   def wrapper( *args, **kwargs ) :
      try :
         return f( *args, **kwargs )
      except BrokenPipeError :
         pass
      except SystemExit :
         raise
      finally :
         # --------------------------------------------------------------------------
         # --- Save origine sys.stdout & sys.stderr to restore after flushing ...
         try    : _stdOut = sys.stdout
         except : _stdOut = False
         try    : _stdErr = sys.stderr
         except : _stdErr = False

         try :
            sys.stdout.flush()
         finally :
            try :
               sys.stdout.close()
               if _stdOut : sys.stdout = _stdOut
            finally :
               try :
                  sys.stderr.flush()
               finally :
                  try :
                     sys.stderr.close()
                     if _stdErr : sys.stderr = _stdErr
                  except :
                     pass

   return wrapper

# -----------------------------------------------------------------------------------
# --- Class to 'escape' Error massages ...
# -----------------------------------------------------------------------------------
class ErrorLog :

   def __init__( self, stream, log_file, buffer=1, iDebug=False ) :
      self.iDebug = iDebug
      self.stream = stream
      self.log    = open( log_file, 'a', buffer, encoding='utf-8' )

   def write( self, msg ) :
      msg = escape( msg )
      self.stream.write( msg )
      if not self.iDebug :
         self.log.write( msg )

   def flush( self ) :
      self.stream.flush()
      if not self.iDebug :
         self.log.flush()

   def __del__( self ) :
      try    : self.log.close()
      except : pass

# -----------------------------------------------------------------------------------
# --- Class : An extended format string formatter :
# --- Formatter with extended conversion symbol
# -----------------------------------------------------------------------------------
class ExtendedFormatter( Formatter ) :

   def convert_field( self, value, conversion ) :

      """ Extend conversion symbol
      Following additional symbol has been added :
      * l: convert to string and low case
      * u: convert to string and up case

      default's are:
      * s: convert with str()
      * r: convert with repr()
      * a: convert with ascii()
      """

      if   conversion == "u" :
         return str( value ).upper()
      elif conversion == "l" :
         return str( value ).lower()
      # --- Do the default conversion or raise error if no matching conversion found
      super( ExtendedFormatter, self ).convert_field( value, conversion )

      # --- return for None case
      return value

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
# --- A Set of Basic File-IO functions ...
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------

# -----------------------------------------------------------------------------------
# --- Class to restrict folder and file searches to existing shares.
# -----------------------------------------------------------------------------------
class ShareTree :

   # --------------------------------------------------------------------------------
   def __init__( self, isRoot=False ) :
      self.shares  = dict()
      self.Root    = isRoot

      # -----------------------------------------------------------------------------
      if self.Root : self.shares['AvrLogger'] = asset.avrPresetFiles

      # -----------------------------------------------------------------------------
      for sVolume in glob.glob( '/volume?' ) :
         for sShare in self.GetFolders( sVolume, clean=False ) :
            self.shares.update( { sShare.split( '/' )[-1] : sShare } )

   # --------------------------------------------------------------------------------
   def GetFolders( self, sPath, clean=True ) :
      aList = []

      if clean and not self.Root : sPath = self.CleanPath( sPath )

      if sPath :
         for sID in glob.glob( os.path.join( sPath, '[!@]*' ) ) :
            if '#recycle' in sID : continue

            if os.path.isdir( sID ) :
               aList.append( sID )
      else :
         for sID in self.shares :
            aList.append( self.shares[sID] )

      return sorted( aList )

   # --------------------------------------------------------------------------------
   def FindeShare( self, sPath ) :

      sHome = sPath.strip().strip( '/' )
      sPath  = self.CleanPath( sPath )

      volume = '/{0}'.format( sPath.split( '/' )[1] )
      share  = '/'.join( sPath.split( '/')[2:] )
      return volume, share

   # --------------------------------------------------------------------------------
   def PathExist( self, sPath, isFile=False, nice=False ) :

      if isFile : 
         sHome = os.path.dirname ( sPath.strip().strip( '/' ) )
         sName = os.path.basename( sPath.strip().strip( '/' ) )
      else :
         sHome = sPath.strip().strip( '/' )

      Home = self.CleanPath( sHome )

      if not Home and self.Root : Home = sHome

      # -----------------------------------------------------------------------------
      if not Home :
         if nice : return False, False, sHome, sHome
         else    : return False, False

      # -----------------------------------------------------------------------------
      if isFile : Load = os.path.isfile( '{0}/{1}'.format( Home, sName ) )
      else      : Load = os.path.isdir ( '{0}'.format( Home ) )

      Save = Home.startswith( '/volume' ) and os.path.isdir( '{0}'.format( Home ) )

      if Home.startswith( '/volume' ) and '/' in Home :
         Nice = '/'.join( Home.split( '/' )[2:] )
      else :
         Nice = 'AvrLogger/Layout'

      if isFile :
         Nice += '/' + sName
         Home += '/' + sName

      if nice : return Load, Save, Home, Nice
      else    : return Load, Save

   # --------------------------------------------------------------------------------
   def GetFiles ( self, sPath, mask='*', clean=True, home=False ) :
      aList = []

      if clean and not self.Root : sPath = self.CleanPath( sPath )

      if sPath :
         for sID in glob.glob( os.path.join( sPath, mask ) ) :
            if os.path.isfile( sID ) :
               aList.append( sID )

      return sorted( aList )

   # --------------------------------------------------------------------------------
   def GetShares( self ) :
      """
      Shares = { **self.shares }

      if home :
         sID = list(home)[0]
         Shares[sID] = home[sID]

      return Shares
      """
      return self.shares

   # --------------------------------------------------------------------------------
   def CleanPath( self, sPath ) :

      sDir = sPath.strip().strip('/')

      if not sDir :
         if self.Root :
            sDir = asset.avrPresetFiles
         else :
            if 'homes' in self.shares :
               sDir = self.shares['homes'].strip( '/' ).split( '/', 1 )[1]
            else :
               sDir = sorted( self.shares )[0]

      if self.Root and 'var/packages' in sDir :
         return asset.avrPresetFiles

      if 'volume' in sDir.split( '/', 1 )[0] :
         try :
            sDir = sDir.split( '/', 1 )[1]
         except :
            sDir = False

      if not sDir or sDir.split( '/', 1 )[0] not in self.shares : return False

      if '/' in sDir : return os.path.join( self.shares[sDir.split( '/', 1 )[0]], sDir.split( '/', 1 )[-1] )
      else           : return self.shares[sDir.split( '/', 1 )[0]]

   # --------------------------------------------------------------------------------
   def GetContent( self, sPath, sMask='*' ) :
      Folder = []
      Files  = []

      vPath  = self.CleanPath ( sPath )
      Shares = self.GetShares ()
      if vPath :
         Folder = self.GetFolders( vPath,        clean=False )
         Files  = self.GetFiles  ( vPath, sMask, clean=False )
      else :
         vPath = '/volume?'

      return ( { 'Folders' : Folder, 'Files' : Files, 'Shares' : Shares, 'fullPath' : vPath } )

# -----------------------------------------------------------------------------------
# --- Check if File exists ...
# -----------------------------------------------------------------------------------
def check( sFile ) :

   if os.path.exists( sFile ) : return True
   else :                       return False


# -----------------------------------------------------------------------------------
# --- Remove (delete) a Directory "recursive" including any content ...
# -----------------------------------------------------------------------------------
def rmTree ( sPath ) :

   shutil.rmtree( sPath )

# -----------------------------------------------------------------------------------
# --- Move "File" Source to Destination ...
# -----------------------------------------------------------------------------------
def ioMove( sSrc, sDest ):

   shutil.move( sSrc , sDest )

   # --------------------------------------------------------------------------------
   if asset.f_Dsm7 :
      try            : shutil.chown ( sDest, user='AvrLogger', group='AvrLogger' )
      except OSError : pass

# -----------------------------------------------------------------------------------
# --- Copy "File" Source to Destination ...
# -----------------------------------------------------------------------------------
def ioCopy( sSrc, sDest, opt=True ):

   shutil.copy( sSrc , sDest )

   # --------------------------------------------------------------------------------
   if asset.f_Dsm7 and opt :
      try            : shutil.chown ( sDest, user='AvrLogger', group='AvrLogger' )
      except OSError : pass

# -----------------------------------------------------------------------------------
# --- Read (cat) a File ...
# -----------------------------------------------------------------------------------
def ioRead( sFile ):

   if os.path.exists( sFile ):
      with open( sFile, 'r', encoding='utf-8' ) as f:
         return f.read().strip()

   return False

# -----------------------------------------------------------------------------------
# --- Write into File (this overwrites an existing file) ...
# -----------------------------------------------------------------------------------
def ioWrite( sText, sFile, opt=True ):

   with open( sFile, 'w+', encoding='utf-8' ) as f:
      f.write( sText )

   # --------------------------------------------------------------------------------
   if asset.f_Dsm7 and opt :
      try            : shutil.chown ( sFile, user='AvrLogger', group='AvrLogger' )
      except OSError : pass

# -----------------------------------------------------------------------------------
# --- Remove (Delete) a File ...
# -----------------------------------------------------------------------------------
def ioRm( sFile ):

   with contextlib.suppress( FileNotFoundError ) :
      os.remove( sFile )

# -----------------------------------------------------------------------------------
# --- touch - equivalent of then bash function ...
# -----------------------------------------------------------------------------------
def touch( sFile, mode=0o755, dir_fd=None, **kwargs ) :

   flags = os.O_CREAT | os.O_APPEND

   with os.fdopen( os.open( sFile, flags=flags, mode=mode, dir_fd=dir_fd ) ) as f :
      os.utime( f.fileno() if os.utime in os.supports_fd else sFile, dir_fd=None if os.supports_fd else dir_fd, **kwargs )

   # --------------------------------------------------------------------------------
   if asset.f_Dsm7 :
      try            : shutil.chown ( sFile, user='AvrLogger', group='AvrLogger' )
      except OSError : pass

# -----------------------------------------------------------------------------------
# --- Get TimeStamp of a File ...
# -----------------------------------------------------------------------------------
def ioStat( sFile ):

   if os.path.exists( sFile ):
      try    : return int( os.path.getmtime( sFile ) )
      except : return False
   else:
      return False

# -----------------------------------------------------------------------------------
# --- Add (write) Content into a File ...
# -----------------------------------------------------------------------------------
def ioAdd( sText, sFile ):

   with open( sFile, 'a+', encoding='utf-8' ) as f:
      f.write( sText + "\n" )

   # --------------------------------------------------------------------------------
   if asset.f_Dsm7 :
      try            : shutil.chown ( sFile, user='AvrLogger', group='AvrLogger' )
      except OSError : pass

# -----------------------------------------------------------------------------------
# --- Add (write) Content into a File (including a TimeStamp) ...
# -----------------------------------------------------------------------------------
"""
def ioLog( sText, sFile ):

   with open( sFile, 'a+', encoding='utf-8' ) as f:
      f.write( time.strftime('%Y.%m.%d %H:%M:%S ') + sText + '\n' )

   # --------------------------------------------------------------------------------
   if asset.f_Dsm7 :
      try            : shutil.chown ( sFile, user='AvrLogger', group='AvrLogger' )
      except OSError : pass
"""
# --------------------------------------- END ---------------------------------------
# -----------------------------------------------------------------------------------
#  sys.exit()
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
