# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#

import os
import sys
import json
import time
import glob

import multiprocessing
import subprocess
import shutil

from   html   import escape
import hashlib

import asset
import basics  as io

# import __main__

escInv = '\033[7m'   # Inverse
escRst = '\033[0m'   # Reset


# --------------------------------------------------------------------------
# --- echo ERROR-Messages to $ERROR or Terminal
# --------------------------------------------------------------------------
# --- $1 = Message- / Outputstring
# --------------------------------------------------------------------------
def Error( sText, opt=False ):

   try :
      if sys.stdout.isatty() and os.environ.get( 'TERM', 'dumb' ) != 'dumb' :
         if opt : print ( sText )
         else   : Echo ( 4, sText )
         return
      else :
         with open( asset.ERROR, 'a+', encoding='utf-8' ) as f:
            f.write( time.strftime('%Y.%m.%d %H:%M:%S ') + escape( sText ) + '\n' )
   except :
      with open( asset.ERROR, 'a+', encoding='utf-8' ) as f:
         f.write( time.strftime('%Y.%m.%d %H:%M:%S ') + escape( sText ) + '\n' )

# --------------------------------------------------------------------------
# --- echo Messages to Terminal
# --------------------------------------------------------------------------
# --- $1 = Message- / Outputstring
# --------------------------------------------------------------------------
def Echo( opt, sText ):

   # -----------------------------------------------------------------------
   if not sys.stdout.isatty() : return

   # -----------------------------------------------------------------------
   global escRst
   global escInv

   while switch( opt ) :
      if case( 0 ) :
         msg = ( f'{sText:88.88}' )
         break
      if case( 1 ) :
         msg = ( f'{sText:79.79}{asset.timerLap:8}|' )
         break
      if case( 2 ) :
         msg = ( f'| {sText:84.84} |' )
         break
      if case( 3 ) :
         msg = escInv + ( "%-86.86s%2s" % ( "| " + sText, "|" ) ) + escRst
         break
      if case( 4 ) :
         msg = escInv + ( "%-86.86s%2s" % ( "| SYS: " + sText, "|" ) ) + escRst
         break
      if case( 5 ) :
         msg = ( "%-86.86s%2s" % ( "| SYS: " + sText, "|" ) )
         break
   # -----------------------------------------------------------------------
      msg = sText
      break

   # --------------------------------------------------------------------------
   print ( msg )

# -----------------------------------------------------------------------------------
# --- 
# -----------------------------------------------------------------------------------
def MakeHash( sText=False, Hash='md5', isFile=False ) :

   if sText : sText = sText.encode( 'utf-8' )

   if isFile :
      with open( isFile, 'rb' ) as f :
         sText = f.read()

   while switch( Hash ) :
      if case( 'md5'    ) :
         return hashlib.md5   ( sText ).hexdigest()
      if case( 'sha1'   ) :
         return hashlib.sha1  ( sText ).hexdigest()
         break
      if case( 'sha256' ) :
         return hashlib.sha256( sText ).hexdigest()
         break
         # --- Default's ----------------------------------------------------
      return False
      break

# -----------------------------------------------------------------------------------
# --- Separate Date, Time into Dict() ...
# -----------------------------------------------------------------------------------
def dictGrepX( sDate=False, sTime=False, sKey=False ) :

   # --------------------------------------------------------------------------------
   if not sKey :
      Error( 'GrepX : A search string is missing' )
      return False

   # --------------------------------------------------------------------------------
   sYear  = False
   sMonth = False
   sDay   = False

   if sDate :
      try    : sYear, sMonth, sDay = sDate.split( '.' )
      except : pass

   if sDate :
      if not sYear :
         Error ( 'GrepX : Date is in the wrong format : [{0}]'.format( sDate ) )
         return False

      if len( sYear) != 4 or len( sMonth ) != 2 or len( sDay ) != 2 :
         Error ( 'GrepX : Date is in the wrong format : [{0}]'.format( sDate ) )
         return False

   # --------------------------------------------------------------------------------
   sHour  = False
   sMin   = False

   if sTime :
      try    : sHour, sMin = sTime.split( ':', 1 )
      except : pass

   if sTime :
      if not sHour :
         Error ( 'Time is in the wrong format : [{0}]'.format( sTime ) )
         return False

      if len( sHour ) != 2 or len( sMin ) >= 3 :
         Error ( 'Time is in the wrong format : [{0}]'.format( sTime ) )
         return False

   # --------------------------------------------------------------------------------
   return   {
               'year'   : sYear,
               'month'  : sMonth,
               'day'    : sDay,
               'hour'   : sHour,
               'minute' : sMin,
               'string' : sKey,
            }

# -----------------------------------------------------------------------------------
# --- Search for a keyword in all log files. Optionally limit the search to date and time ...
# -----------------------------------------------------------------------------------
def LogGrepX( sDate=False, sTime=False, sKey=False ) :

   sTerm = dictGrepX( sDate=sDate, sTime=sTime, sKey=sKey )

   if not sTerm : return []

   # --------------------------------------------------------------------------------
   if not sDate and not sTime :
      # --- Just only search string :
      sGrep = ".*'{string}').*'".format( **sTerm )

   elif sDate and not sTime :
      # --- Only Date :
      sGrep = "'{year}'.?'{month}'.?'{day}'.*'{string}').*'".format( **sTerm )

   elif not sDate and sTime :
      # --- Only Time :

      if sTerm[ 'minute' ] == '--' :
         # --- Time only hour
         sGrep = "'{hour}':.{0}.*'{string}').*'".format( '{2}:.{2}', **sTerm )
      else :
         # --- Time : hour and minute(s)
         if len ( sTerm[ 'minute' ] ) == 1 :
            sGrep = "'{hour}':'{minute}'.{0}.*'{string}').*'".format( '{1}:.{2}', **sTerm )
         else :
            sGrep = "'{hour}':'{minute}':.{0}.*'{string}').*'".format( '{2}', **sTerm )
   elif sDate and sTime :
      # --- Both, Date and Time :
      if sTerm[ 'minute' ] == '--' :
         sGrep = "'{year}'.?'{month}'.?'{day}'.?'{hour}':.{0}.*'{string}').*'".format( '{2}:.{2}', **sTerm )
      else :
         if len ( sTerm[ 'minute' ] ) == 1 :
            sGrep = "'{year}'.?'{month}'.?'{day}'.?'{hour}':'{minute}'.{0}.*'{string}').*'".format( '{1}:.{2}', **sTerm )
         else :
            sGrep = "'{year}'.?'{month}'.?'{day}'.?'{hour}':'{minute}':.{0}.*'{string}').*'".format( '{2}', **sTerm )

   # --------------------------------------------------------------------------------
   # --- Prepare Find command ...
   sCMD = BuildFindCmd( sPath='/var/log', exFile=[ '*.xz', '*.txz', '*.csv', '*.json', '.*', '..*', '*.xml', '*.result' ] )
   iRet = getCMD( sCMD + " -exec grep -EnH '.*(" + sGrep + " {} \;" ).strip().split( '\n' )

   if iRet[0] : return iRet
   else       : return []

# -----------------------------------------------------------------------------------
# --- Get File-List or prepare "find" cmd ...
# -----------------------------------------------------------------------------------
def BuildFindCmd( sPath='', exPath=[], exFile=[], sMask=False, iLimit=False ) :

   # --- Start find-command ...
   if iLimit : sCMD = f'find {sPath} -follow -maxdepth {iLimit}'
   else      : sCMD = f'find {sPath}'

   # --- Add 'exclude' Path(s) ...
   for msk in exPath :
      sCMD += f' -not -path "*{msk}*"'

   # --- Add search 'file' Mask ...
   if sMask : sCMD += f' -type f -name "{sMask}"'
   else     : sCMD +=  ' -type f -name "*"'

   # --- Add 'exclude' File(s) ...
   for msk in exFile :
      sCMD += f' ! -name "{msk}"'

   # --- Excecute 'find'-Command ...
   return sCMD

# -----------------------------------------------------------------------------------
# --- Performe an Url-Request (cURL) :
# -----------------------------------------------------------------------------------
"""
def UrlReqest( request, msg ) :
   import urllib.request as     Request
   from   urllib.error   import URLError

   try:
      res = Request.urlopen( request )
      if res.getcode() == 200 :
         return str( res.read(), "utf-8" )
      else :
         return str( res.getcode() )

   except URLError as e:
      return str( e.code )

   except Exception as e :
      Error( 'CLI: UrlRequest \"{0}\" faild.'.format( msg ) )
      Error( 'CLI: Exception = \"{0}\"'.format( e ) )
      return '500'
"""

# -----------------------------------------------------------------------------------
# --- Return USB Vendor
# -----------------------------------------------------------------------------------
def FindUsbVendor( vID ) :

   sRet = UsbVendors.get( vID.upper() )
   if sRet : return sRet
   else    : return 'NA'

# -----------------------------------------------------------------------------------
# --- Return USB Product by Vendor
# -----------------------------------------------------------------------------------
def FindUsbProduct( vID, pID ) :

   sRet = UsbProducts.get( ( vID.upper(), pID.upper() ) )
   if sRet : return sRet
   else    : return 'NA'

# -----------------------------------------------------------------------------------
# --- Return USB protocol from usbclasses list ...
# -----------------------------------------------------------------------------------
def FindUsbClass( cID, sID, pID ) :

   sRet = UsbClasses.get( ( cID,  sID,  pID ) ) \
      or  UsbClasses.get( ( cID,  sID, 'FF' ) ) \
      or  UsbClasses.get( ( cID, 'FF', 'FF')  )

   if sRet : return sRet
   else    : return '(unknown)'

# -----------------------------------------------------------------------------------
# --- return True if all digits are valid hex digits ...
# -----------------------------------------------------------------------------------
def isHexDigit( sHex ) :

   for sChr in sHex :
      if not sChr.isdigit() and not sChr in 'aAbBcCdDeEfF' : return False

   return True

# -----------------------------------------------------------------------------------
# --- Parse /usr/share/usb.ids and fill UsbVendors, UsbProducts, UsbClasses ...
# -----------------------------------------------------------------------------------
iUsbIDs        = [ '/usr/syno/etc/usb.ids',
                   '/usr/syno/etc/usb.map',]

UsbVendors     = dict()
UsbProducts    = dict()
UsbClasses     = dict()

def ParseUsbIDs( opt=False ) :

   vid      = ''
   did      = ''
   mode     = 'Vendor'
   strg     = ''
   cstrg    = ''

   sID      = False

   # --------------------------------------------------------------------------------
   # --- Search for usb.ids ...
   for sFile in iUsbIDs :
      if not os.path.exists( sFile ) : continue

      sID = True

   # --------------------------------------------------------------------------------
   if not sID : return False

   # --------------------------------------------------------------------------------
   with open( sFile, 'r', encoding='utf-8' ) as f:
      for line in f:
         # --------------------------------------------------------------------------
         # --- Parse usb.ids line by line ...
         ln = line.rstrip()

         # --------------------------------------------------------------------------
         # --- Ignore blank or comment lines ...
         if not ln or ln[0] == '#' :
            continue

         # --------------------------------------------------------------------------
         if isHexDigit( ln[0:4].upper() ) :
            mode              = 'Vendor'
            vid               = ln[:4].upper()
            UsbVendors[ vid ] = ln[6:].strip()
            continue

         # --------------------------------------------------------------------------
         if ln[0] == '\t' and isHexDigit( ln[1:5].upper() ) :
            did = ln[1:5].strip().upper()

            # -----------------------------------------------------------------------
            if mode == 'Vendor' :
               UsbProducts[ vid, did ] = ln[7:].strip()
               continue
            elif mode == 'Class' :
               nm = ln[5:]
               if nm != 'Unused' :
                  strg = cstrg + ':' + nm
               else :
                  strg = cstrg + ':'

               UsbClasses[ vid, did, 'FF' ] = strg
               continue

         # --------------------------------------------------------------------------
         if ln[0] == 'C' :
            mode = 'Class'
            cid = ln[2:4].strip().upper()
            cstrg = ln[6:].strip()
            UsbClasses[ cid, 'FF', 'FF' ] = cstrg
            continue

         # --------------------------------------------------------------------------
         if mode == 'Class' and ln[0] == '\t' and ln[1] == '\t' and isHexDigit( ln[2:4].upper() ) :
            prid = ln[2:4].strip().upper()
            UsbClasses[cid, did, prid] = ln[6:].strip()
            continue

         mode = 'Misc'

      # -----------------------------------------------------------------------------
      UsbClasses[ 'FF', 'FF', 'FF' ] = 'Vendor Specific'

   # --------------------------------------------------------------------------------
   return True

# -----------------------------------------------------------------------------------
# --- Determine "shared memory" status ...
def GetShm() :

   stat = getCMD('df -BM | grep -m 1 "shm"').strip()
   if stat and not os.path.exists( asset.f_ExcludeSHM ) :
      stat = " ".join(stat.split()).split(' ')
      if int(stat[3][:-1]) >= 5 :
         return True

   return False

# ------------------------------------------------------------------------------
# --- range as Float ...
def xrange( start, stop, step, opt='<' ) :

   if step > 0 :
      if opt == '<=' :
         while round( start, 3 ) <= stop :
            yield round( start, 3 )
            start += step
         return
      else :
         while round( start, 3 ) < stop :
            yield round( start, 3 )
            start += step
         return
   else :
      if opt == '>=' :
         while round( start, 3 ) >= stop :
            yield round( start, 3 )
            start += step
         return
      else :
         while round( start, 3 ) > stop :
            yield round( start, 3 )
            start += step
         return

# ------------------------------------------------------------------------------
# --- Trenne einen String ...
def Split( sText ):

   if '" "' in sText : return sText.strip('"').split('" "')
   else:               return sText.split(' ')

# --------------------------------------------------------------------------
# --- Get Time-Lap in (ms) nanoseconds ...
# --------------------------------------------------------------------------
def TimerStart() :

   try:    asset.timerStart[asset.timerCount] = time.time()
   except: asset.timerStart.append( time.time() )

   try:    asset.timerCount += 1
   except: asset.timerCount  = 1

# --------------------------------------------------------------------------
# --- Write passed time since TimerStart in timerLap (milliseconds)
# --------------------------------------------------------------------------
def TimerStop( opt=False ):

   asset.timerCount -= 1
   if not opt : asset.timerLap = ( f'{time.time() - asset.timerStart[asset.timerCount]:.5f}' )
   else       : return           ( f'{time.time() - asset.timerStart[asset.timerCount]:.5f}' )

# --------------------------------------------------------------------------
# --- 
# --------------------------------------------------------------------------
def unset( sDict, sKey ):

   if sKey in sDict : sDict.pop( sKey )

# --------------------------------------------------------------------------
# --- 
# --------------------------------------------------------------------------
def Case( site, more ):
   return ''.join(( site, '-', more ))

# -----------------------------------------------------------------------------------
# --- Find "share" : Clean up / correct the path
# -----------------------------------------------------------------------------------
# --- $1 = share-path
# --- >>> Return Array if found:
# -->    arg[0] = Corrected "share" path
# -->    arg[1] = Volume of "share" path
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
def FindeShare( sPath ):

   UserDir = sPath.strip()
   UserDir = UserDir.strip('/')

   if 'volume' in UserDir.split( '/', 1 )[0] :
      UserDir = UserDir.split( '/', 1 )[1]

   if not UserDir : return

   for entry in os.scandir( '/' ):
      if entry.is_dir() and 'volume' in entry.path :
         if os.path.exists( entry.path + "/" + UserDir ):
            return entry.path, UserDir

# --------------------------------------------------------------------------
# --- Nachbildung einer "switch / case" Anweisung ...
# --------------------------------------------------------------------------
class switch(object):
    value = None
    def __new__( class_, value ) :
        class_.value = value
        return True

def case(*args):
    return any(( arg == switch.value for arg in args ))

"""
#  Beispiel :
   while switch( iAxis ) :
      if case( 2 ) :
         if svg.Eth_Flag :
            svg.LegColCount  = 0
            svg.LegRowCount += 1
            svg.Eth_Flag     = False
            break
      if case( 3 ) :
         if svg.Md_Flag :
            svg.LegColCount  = 0
            svg.LegRowCount += 1
            svg.Md_Flag      = False
            break
      break
"""

# --------------------------------------------------------------------------
# --- Get PID(s) by Name :
# --- Examine every ''cmdline'' to see if ''Name'' is included.
# --------------------------------------------------------------------------
def GetPPidOf( sName, opt=False ) :

   PID = []
   for sDir in os.listdir( '/proc' ) :
      if sDir.isdigit() :
         try:
            if sName in open( f'/proc/{sDir}/cmdline' ).read() :
               PID.append( int( sDir ) )
         except:
            continue

   return PID

# --------------------------------------------------------------------------
# --- Call a "DSM" function and deliver the result ...
# --------------------------------------------------------------------------
def getCMD( cmd, shell=True, NoErr=True  ):

   try:
      out, err = subprocess.Popen(cmd, shell=shell, stdout=subprocess.PIPE, stdin=subprocess.PIPE ).communicate()
      if NoErr : return      out.decode("utf-8")
      else     : return err, out.decode("utf-8")
   except:
      Error( 'SYS: "getCMD" failed to start Subprocess :' )
      Error( 'SYS: cmd was : {0}'.format( cmd ) )
      return False

# --------------------------------------------------------------------------
# --- Start an asynchronous "task" ...
# --------------------------------------------------------------------------
def CallTask( sTrade, sArg=False ) :

   try :
      if not sArg: PID = multiprocessing.Process( target=sTrade )
      else:        PID = multiprocessing.Process( target=sTrade, args=( sArg, ) )

      PID.daemon = True
      PID.start()
      return PID
   except:
      Error( 'SYS: "CallTask" failed to start Process :' )
      Error( 'SYS: Arg was : {0}'.format( sArg ) )
      return False

# -----------------------------------------------------------------------------------
# --- Send a notification to DSM ...
# -----------------------------------------------------------------------------------
def Note( className=False, sUser=False, sSection='Notify', sTitel='Titel', sText='Oops', sWho=False, i18n=False, sCustom=False ) :

   if not className : className = 'SYNO.SDS.AvrLogger.Application'
   if not sUser     : sUser     = '@administrators'
   if not sWho      : sWho      = asset.SpkName
   # --------------------------------------------------------------------------------
   # --- Add a meassage to Protocoll-Center :
   # --- /usr/syno/bin/synologset1 sys info 0x11800000 "$0: SDK Unmounted"
   # --- Copyright (c) 2003-2021 Synology Inc. All rights reserved.
   # --- USAGE : synologset1 [sys | man | conn]  [info | warn | err]  eventID(%X)  [substitution strings...]
   # --- ***** : eventID is an unsigned long and should be inserted as 8 digits, big case string.

   # --------------------------------------------------------------------------------
   # --- If an i18n string is given but no DSM 7 ... create downward compatibility ...
   if i18n and not asset.f_Dsm7 :
      i18n = False
               
      msg = InitLanguage( sName=False, sPath=asset.i18nLangFile, suffix='', i18n=True )
      sTitel = msg[ sSection ][ sTitel ]
      sText  = msg[ sSection ][ sText  ]

   if sCustom : sCustom = f' "{sCustom}"'
   else       : sCustom = ''

   # --------------------------------------------------------------------------------
   # --- Send a DSM notification ...
   if i18n : getCMD( f'synodsmnotify -c {className} {sUser} {sWho}:{sSection}:{sTitel} {sWho}:{sSection}:{sText}{sCustom}' )
   else    : subprocess.call( [ 'synodsmnotify', sUser, sTitel, sText ] )

# -----------------------------------------------------------------------------------
# --- Get System / User language ...
# -----------------------------------------------------------------------------------
def InitLanguage( sName=False, sPath='', sLang=False, suffix='.sh', opt=False, i18n=False, iOnly=False, Debug=False ) :

   # --------------------------------------------------------------------------------
   # --- Get current user ...
   if not sName :
      try :
         sysCookies = json.loads( io.ioRead( asset.cgiSynoCheck ) )
         sName      = sysCookies['SynoUser']
      except :
         sName      = 'admin'

   # --------------------------------------------------------------------------------
   # --- First try to get user language ...
   try :
      Data = json.loads( io.ioRead( f'/usr/syno/etc/preference/{sName}/usersettings' ) )
      lang  = Data['Personal']['lang']
   except :
      lang = False

   if not lang or lang in [ 'def', 'null' ] :
      # -----------------------------------------------------------------------------
      # --- Get system language ...
      if not bool( asset.SynoInfo ) : asset.SynoInfo = FileConfig( asset.sysSynoConfig )

      lang = asset.SynoInfo.get( 'language', False )

      if not lang or lang == 'def':
         lang = 'enu'

   # --------------------------------------------------------------------------------
   if iOnly : return lang

   # --------------------------------------------------------------------------------
   # --- Check if a suitable language file is available ...
   if i18n :
      file = f'{sPath}{lang}/strings'
      if not os.path.exists( file ) : file = f'{sPath}enu/strings'

   else :
      file = f'{sPath}{lang}{suffix}'
      if Debug : print ( f'File     : {file}' )
      if not os.path.exists( file ) : file = f'{sPath}enu{suffix}'

   if Debug : print ( f'File     : {file}' )

   # --------------------------------------------------------------------------------


   if opt:
      Lang  = io.ioRead( file )
   else:
      Lang = dict()
      Name = False
      ID = open( file, encoding='utf-8' ).read().split('\n')

      if i18n :
         # --------------------------------------------------------------------------
         # --- Read i18n file and convert it ...
         for line in ID :

            if not line       : continue
            if line[0] == '#' : continue

            if not Name and line[0] != '[' : continue

            if line[0] == '[' and line[-1] == ']' :
               Name = line[1:-1]
               Lang[ Name ] = dict()
               continue

            sKey, sValue = line.split( '=', 1 )
            sKey   = sKey.strip()
            sValue = sValue.strip()
            Lang[ Name ][ sKey ] = sValue[1:-1]

      else :
         # --------------------------------------------------------------------------
         # --- Read language file and clean up ...
         for line in ID:
            if line:
               key,value=line.split( '=', 1 )
               value = value.replace( '\\', '' )
               Lang[key]=value[1:-1]

   # --------------------------------------------------------------------------------
   return Lang

# --------------------------------------------------------------------------
# --- Ein "Token" in einem File suchen und dessen Wert liefern ...
# --------------------------------------------------------------------------
def GetValueFile(sName, sPath):
   Token = ""

   ID = open( sPath, encoding='utf-8' ).read().split( '\n' )
   for line in ID:
      if sName in line:
         Token = line.strip()
         Token = Token.split( '=', 1 )[1]
         Token = Token.strip( '"' )
         break

   return Token

# --------------------------------------------------------------------------
# --- Die "Avr-NG" Konfig-Datei mit aktualisierten Werten zurückschreiben
# --- und eine Kopie ertellen.
# --------------------------------------------------------------------------
def ConfigSave() :

   try    : AvrConf = asset.AvrConf
   except : return

   # -----------------------------------------------------------------------
   ID = open( asset.avrConfigFile ).read().split( '\n' )
   tmp = ""

   for line in ID :
      if line :
         key  = line.split( '=', 1 )[0]
         tmp += key + '=\"' + AvrConf[key] + "\"\n"

   io.ioWrite( tmp, asset.avrConfigFile )
   io.ioCopy ( asset.avrConfigFile, asset.avrConfigFile + '.save' )
   # -----------------------------------------------------------------------

   return

# --------------------------------------------------------------------------
# --- Read a "cleard" Config-File ...
# --------------------------------------------------------------------------
def ConfigRead( sPath ):

   Conf = dict()

   for sLine in open( sPath, encoding='utf-8' ).read().split('\n')[:-1] :
      sKey, sValue = sLine.split( '=', 1 )
      Conf[ sKey]  = sValue[1:-1]

   return Conf

# -----------------------------------------------------------------------------------
# --- Read Config-File and return result as dict ...
# -----------------------------------------------------------------------------------
def FileConfig( sPath, Conf=dict() ) :

   iList = open( sPath, encoding='utf-8', errors='ignore' ).read().split('\n')

   for line in iList:
      if not line or line[0] == '#' : continue

      try    : key, value = line.split( '=', 1 )
      except : continue

      key   = key.strip()
      value = value.strip()

      if value[0] == '"' and value[-1] == '"' :
         value = value.strip( '"' )

      Conf[key]=value

   return Conf

# -----------------------------------------------------------------------------------
# --- System "snmpwalk v2" Call :
# -----------------------------------------------------------------------------------
def WalkV2( sCom, sIp, sOID ):

   cmd = 'snmpwalk -r 0 -t 1 -v 2c -c "{0}" -O qv "{1}" "{2}" 2>&1'.format( sCom, sIp, sOID )
   return getCMD( cmd ).strip()

# -----------------------------------------------------------------------------------
# --- System "snmpwalk v3" Call :
# -----------------------------------------------------------------------------------
def WalkV3( sUser, sEnc, sPass, sIp, sOID ):

   cmd = 'snmpwalk -r 0 -t 1 -v 3 -l authNoPriv -u "{0}" -a "{1}" -A "{2}" "{3}" -O qv "{4}" 2>&1'.format( sUser, sEnc, sPass, sIp, sOID )
   return getCMD( cmd ).strip()

# --------------------------------------- END ---------------------------------------
# -----------------------------------------------------------------------------------
#  sys.exit()
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
